{% extends "moneta/base_package.html" %}{% load bootstrap3 i18n moneta %}
{% block bootstrap3_title %}{% trans 'Search repository' %}{% endblock %}

{% block repo_content %}

                <form action="?" method="get" class="form" role="form">
                    {% bootstrap_form form %}
                    <input type="submit" class="btn btn-success" value="{% trans 'Search' %}" />
                </form>
                {% bootstrap_pagination_extra elements search=pattern %}
                <ul class="nav nav-pills">
                    {% for element in elements %}
                        <li>{% include 'moneta/element.html' %}</li>
                    {% empty %}
                        <li><p class="well">{% trans 'No package was found.' %}</p></li>
                    {% endfor %}
                </ul>
                {% bootstrap_pagination_extra elements search=pattern %}
{% endblock %}

