{% load i18n moneta %}
<h3>{{ element }}</h3>
<ul class="list-unstyled">
    <li>{% trans 'Filesize: ' %} <strong>{{ element.filesize|filesizeformat }}</strong></li>
    <li>{% trans 'States: ' %}<strong>{% for state in element.states.all %}{{ state.name }} {% empty %}{% trans 'No state' %}{% endfor %}</strong></li>
    <li>
        <div class="btn-group">
            <a href="{{ element|direct_link }}" class="btn btn-success btn-xs">
                {% with filename=element.filename %}{% blocktrans %}Download {{ filename }}
                {% endblocktrans %}{% endwith %}</a>
            <a href="{% url 'moneta:show_file' element.id %}" class="btn btn-success btn-xs">{% trans 'More infos…' %}</a>
            <a href="{% url 'moneta:delete_element' rid=repo.id eid=element.id %}" class="btn btn-danger btn-xs">
                {% with filename=element.filename %}{% blocktrans %}Delete {{ filename }}
                {% endblocktrans %}{% endwith %}</a>
        </div>
    </li>
</ul>
