{% extends "moneta/base_package.html" %}{% load staticfiles i18n %}
{% block bootstrap3_title %}{% blocktrans %}Repository {{ repo }}{% endblocktrans %}{% if state_slug %}/{{ state_slug }}{% endif %}{% endblock %}

{% block repo_content %}
    <h1><a href="{{ repo.get_absolute_url }}">{% blocktrans %}Repository {{ repo }}/{{ state }}{% endblocktrans %}</a></h1>
    <p>{% blocktrans %}Available packages in <strong>{{ repo }}/{{ state }}/{{ folder }}</strong> {% endblocktrans %}
        <span class="badge">{{ element_count }}</span></p>
    <ul class="nav nav-pills">
        {% for element in elements %}
            <li>{% include 'moneta/element.html' %}</li>
        {% endfor %}
    </ul>
{% endblock %}

